;********************************************************
;
;                 Temperature Controller
;
;                                 Device : PIC16F873
;                                 Clock  : 4MHz
;                                 Author : Seiichi Inoue
;********************************************************

        processor       16F873
        include         p16f873.inc
        radix           dec
        __config        h'3F32' ;OSC is HS
                                ;RB3,RB6,RB7 are I/O
                                ;Brown-out detection OFF
                                ;Power-up timer ON
                                ;Code protection OFF
                                ;Data code protection OFF
                                ;Watchdog timer OFF
                                ;LVP OFF
        errorlevel      -302    ;Suppress bank warning

;****************  Label Definition  ********************
        cblock  h'20'
out_temp                        ;Out-side temperature
in_temp                         ;In-side temperature
out10                           ;10th of out-side
out1                            ;1st of out-side
in10                            ;10th of in-side
in1                             ;1st of in-side
preset10                        ;10th of preset
preset1                         ;1st of preset
interval                        ;Interval sw
mode                            ;Mode(1:up 0:down)
test_mode                       ;Test(2:OFF,1:IN,0:OUT)
scan_cont                       ;Scanning conter
rb_cont                         ;RB control data
rc_cont                         ;RC control data
rb_data                         ;RB data
t25us_cnt                       ;25 microseconds counter
t100us_cnt                      ;100 microseconds counter
t1ms_cnt                        ;1 millisecond counter

seg_ha                          ;7seg LED table head adr
seg0                            ;0
seg1                            ;1
seg2                            ;2
seg3                            ;3
seg4                            ;4
seg5                            ;5
seg6                            ;6
seg7                            ;7
seg8                            ;8
seg9                            ;9
sega                            ;A(Blank)
        endc

;****************  Program Start  ***********************
        org     0               ;Reset Vector
        goto    init
        org     4               ;Interrupt Vector
        goto    int

;****************  Initial Process  *********************
init

;*** Port initialization
        bsf     status,rp0      ;Change to Bank1
        movlw   b'00011011'     ;AN0/1/3/RA4:IN RA2/5:OUT
        movwf   trisa           ;Set RA port mode
        clrf    trisb           ;Set RB port output mode
        clrf    trisc           ;Set RC port output mode

;*** Timer 0 initialization (for temp sampling : 10ms)
        movlw   b'00000101'     ;PS=1:64
        movwf   option_reg      ;Set OPTION_REG register
        bcf     status,rp0      ;Change to Bank0
        movlw   100             ;256-10000us/64us=100
        movwf   tmr0            ;Set TMR0

;*** Timer 1 initialization (for hold timer : 500ms)
        movlw   b'00110001'     ;Pre=1:8 TMR1=Int TMR1=ON
        movwf   t1con           ;Set T1CON register
        movlw   h'0b'           ;65536-500000us/8us=3036
        movwf   tmr1h           ;Set TMR1H register
        movlw   h'dc'           ;3036=h'BDC'
        movwf   tmr1l           ;Set TMR1L register

;*** A/D converter initialization
        bsf     status,rp0      ;Change to Bank1
        movlw   b'00000100'     ;ADFM=0 PCFG=0100
        movwf   adcon1          ;Set ADCON1 register
        bcf     status,rp0      ;Change to Bank0

;*** 7seg LED pattern data setting
        movlw   seg0            ;Set table head address
        movwf   seg_ha
        movlw   b'11000000'     ;0
        movwf   seg0
        movlw   b'11111001'     ;1
        movwf   seg1
        movlw   b'10100100'     ;2
        movwf   seg2
        movlw   b'10110000'     ;3
        movwf   seg3
        movlw   b'10011001'     ;4
        movwf   seg4
        movlw   b'10010010'     ;5
        movwf   seg5
        movlw   b'10000010'     ;6
        movwf   seg6
        movlw   b'11111000'     ;7
        movwf   seg7
        movlw   b'10000000'     ;8
        movwf   seg8
        movlw   b'10010000'     ;9
        movwf   seg9
        movlw   b'11111111'     ;A(Blank)
        movwf   sega

;*** Work area initialization
        clrf    porta           ;Clear PORTA
        movlw   b'11111111'
        movwf   portb           ;Set PORTB
        movwf   portc           ;Set PORTC
        movlw   9
        movwf   scan_cont       ;Set scanning counter
        movlw   h'8'            ;Display turning-off
        movwf   out10
        movwf   out1
        movwf   in10
        movwf   in1
        movwf   preset10
        movwf   preset1

;*** Interruption control
        movlw   b'01100000'     ;GIE=ON,PEIE=ON,T0IE=ON
        movwf   intcon          ;Set INTCON register

;************  LED and SW scanning Process  *************
scan
        movfw   scan_cont       ;Read scanning counter
        sublw   9
        btfsc   status,z        ;9 ?
        goto    process1        ;10th of out-side disp
        movfw   scan_cont       ;Read scanning counter
        sublw   8
        btfsc   status,z        ;8 ?
        goto    process2        ;1st of out-side disp
        movfw   scan_cont       ;Read scanning counter
        sublw   7
        btfsc   status,z        ;7 ?
        goto    process3        ;10th of in-side disp
        movfw   scan_cont       ;Read scanning counter
        sublw   6
        btfsc   status,z        ;6 ?
        goto    process4        ;1st of in-side dsip
        movfw   scan_cont       ;Read scanning counter
        sublw   5
        btfsc   status,z        ;5 ?
        goto    process5        ;10th of preset disp
        movfw   scan_cont       ;Read scanning counter
        sublw   4
        btfsc   status,z        ;4 ?
        goto    process6        ;1st of preset disp
        movfw   scan_cont       ;Read scanning counter
        sublw   3
        btfsc   status,z        ;3 ?
        goto    process7        ;10th of preset sw
        movfw   scan_cont       ;Read scanning counter
        sublw   2
        btfsc   status,z        ;2 ?
        goto    process8        ;1st of preset sw
        movfw   scan_cont       ;Read scanning counter
        sublw   1
        btfsc   status,z        ;1 ?
        goto    process9        ;Interval sw

scan1
        movlw   9
        movwf   scan_cont       ;Set scanning counter
        goto    scan

process1
        movlw   b'11111110'
        movwf   rc_cont         ;Set RC control data
        movfw   out10           ;Read 10th of out-side
        movwf   rb_data         ;Set RB data
        call    led_on          ;Call LED ON subroutine
        goto    proc_end

process2
        movlw   b'11111101'
        movwf   rc_cont         ;Set RC control data
        movfw   out1            ;Read 1st of out-side
        movwf   rb_data         ;Set RB data
        call    led_on          ;Call LED ON subroutine
        goto    proc_end

process3
        movlw   b'11111011'
        movwf   rc_cont         ;Set RC control data
        movfw   in10            ;Read 10th of in-side
        movwf   rb_data         ;Set RB data
        call    led_on          ;Call LED ON subroutine
        goto    proc_end

process4
        movlw   b'11110111'
        movwf   rc_cont         ;Set RC control data
        movfw   in1             ;Read 1st of in-side
        movwf   rb_data         ;Set RB data
        call    led_on          ;Call LED ON subroutine
        goto    proc_end

process5
        movlw   b'11101111'
        movwf   rc_cont         ;Set RC control data
        movfw   preset10        ;Read 10th of preset
        movwf   rb_data         ;Set RB data
        call    led_on          ;Call LED ON subroutine
        goto    proc_end

process6
        movlw   b'11011111'
        movwf   rc_cont         ;Set RC control data
        movfw   preset1         ;Read 1st of preset
        movwf   rb_data         ;Set RB data
        call    led_on          ;Call LED ON subroutine
        goto    proc_end

process7
        movlw   b'11111111'
        movwf   portb           ;LED OFF
        bsf     status,rp0      ;Change to Bank1
        movlw   b'01111111'     ;RB0-6:IN RB7:OUT
        movwf   trisb           ;Set RB port mode
        bcf     status,rp0      ;Change to Bank0
        movlw   b'10111111'
        movwf   portc           ;Set PORTC
        call    t100us          ;Wait 100 microseconds
        movfw   portb           ;Read PORTB
        xorlw   b'11111111'     ;Reverse bits
        movwf   preset10        ;Save 10th of preset sw
        movlw   b'00001111'     ;Set mask
        andwf   preset10,f      ;Pick up sw data
        goto    proc_end

process8
        movlw   b'01111111'
        movwf   portc           ;Set PORTC
        call    t100us          ;Wait 100 microseconds
        movfw   portb           ;Read PORTB
        xorlw   b'11111111'     ;Reverse bits
        movwf   preset1         ;Save 1st of preset sw
        movlw   b'00001111'     ;Set mask
        andwf   preset1,f       ;Pick up sw data
        goto    proc_end

process9
        movlw   b'11111111'
        movwf   portc           ;Set PORTC
        bcf     portb,7         ;Set PORTB
        call    t100us          ;Wait 100 microseconds
        movfw   portb           ;Read PORTB
        xorlw   b'11111111'     ;Reverse bits
;        movwf   interval        ;Save interval sw data
        movwf   out1
        movlw   b'00001111'     ;Set mask
;        andwf   interval,f      ;Pick up sw data
        andwf   out1,f
        bsf     status,rp0      ;Change to Bank1
        clrf    trisb           ;Set RB port output mode
        bcf     status,rp0      ;Change to Bank0
        movlw   b'11111111'
        movwf   portb           ;LED OFF

;*** UP/DOWN mode check
        btfss   porta,4         ;RA4 = 1 ?
        goto    proc91          ;No.
        movlw   1
;        movwf   mode            ;Set UP mode
        movwf   in10
        goto    proc92
proc91
;        clrf    mode            ;Set DOWN mode
        clrf    in10

;*** TEST mode check
proc92
        movlw   b'01011001'     ;ADCS=01 CHS=AN3 ADON=ON
        movwf   adcon0          ;Set ADCON0 register
        call    t25us           ;Wait 25 microseconds
        bsf     adcon0,2        ;Start A/D conversion
proc93
        btfsc   adcon0,2        ;Conversion end ?
        goto    proc93          ;No.
        movlw   168             ;Set 3.3V
        subwf   adresh,w        ;AN3 - 3.3V
        btfss   status,c        ;AN3 >= 3.3V
        goto    proc94          ;No.
        movlw   2               ;Yes.
;        movwf   test_mode       ;Set TEST OFF
        movwf   in1
        goto    proc_end
proc94
        movlw   87              ;Set 1.7V
        subwf   adresh,w        ;AN3 - 1.7V
        btfss   status,c        ;AN3 >= 1.7V
        goto    proc95          ;No.
        movlw   1               ;Yes.
;        movwf   test_mode       ;Set IN TEST
        movwf   in1
        goto    proc_end
proc95
;        clrf    test_mode       ;Set OUT TEST
        clrf    in1

proc_end
        decfsz  scan_cont,f     ;Scan end ?
        goto    scan            ;No.
        goto    scan1           ;Yes

;*** LED ON subroutine
led_on
        movlw   b'11111111'
        movwf   portb           ;LED OFF
        movfw   rc_cont         ;Read RC control data
        movwf   portc           ;Set PORTC
        call    t100us          ;Wait 100 microseconds
        movfw   rb_data         ;Read disp digit
        addwf   seg_ha,w        ;HA + Digit
        movwf   fsr             ;Set address
        movfw   indf            ;Read 7seg data
        movwf   portb           ;Set PORTB
        call    t1ms            ;Wait 1 millisecond
        return

;*** 25 microseconds subroutine
t25us
        movlw   5
        movwf   t25us_cnt       ;Set loop counter
t25us_loop
        nop
        decfsz  t25us_cnt,f     ;Counter - 1
        goto    t25us_loop      ;Not end
        return


;*** 100 microseconds subroutine
t100us
        movlw   24
        movwf   t100us_cnt      ;Set loop counter
t100us_loop
        nop
        decfsz  t100us_cnt,f    ;Counter - 1
        goto    t100us_loop     ;Not end
        return

;*** 1 millisecond subroutine
t1ms
        movlw   9
        movwf   t1ms_cnt        ;Set loop counter
t1ms_loop
        call    t100us
        decfsz  t1ms_cnt,f      ;Counter - 1
        goto    t1ms_loop       ;Not end
        return

;***************  Interruption Process  *****************
int
        goto    int_end


;************  END of Interruption Process **************
int_end
        retfie


;********************************************************
;            END of Temperature Controller
;********************************************************
        end
